@@ -1,7 +1,5 @@
 /**
  * @file   strip_mfcc.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 00:38:57 2005
  *
  * <JA>
  * @brief  MFCC󤫤饼ץ
@@ -41,13 +39,16 @@
  * misdetected.  When the misdetection frequently occurs on your MFCC file,
  * please try "-nostrip" option to turn off the stripping.
  *
- * $Revision: 1.4 $
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 00:38:57 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -81,7 +82,7 @@ guess_abs_e_location(HTK_Param *param)
     }
   } else {
     /* absolute energy not included */
-    //j_printerr("Info: absolute energy coef. not found, stripping disabled\n");
+    jlog("Stat: strip_mfcc: absolute energy coef. not exist, stripping disabled\n");
     abs_e_num = -1;
   }
   return abs_e_num;
@@ -108,8 +109,7 @@ param_strip_zero(HTK_Param *param)
   dst = 0;
   for(src=0;src<param->samplenum;src++) {
     if (IS_INVALID_FRAME_MFCC(param->parvec[src][eloc])) {
-      j_printf("Warning: frame %d has invalid energy, stripped\n", src);
-      free(param->parvec[src]);
+      jlog("Warning: strip_mfcc: frame %d has invalid energy, stripped\n", src);
       continue;
     }
     if (src != dst) {
@@ -118,7 +118,7 @@ param_strip_zero(HTK_Param *param)
     dst++;
   }
   if (dst != param->samplenum) {
-    j_printf("Warning: input shrinked from %d to %d frames\n", param->samplenum, dst);
+    jlog("Warning: strip_mfcc: input shrinked from %d to %d frames\n", param->samplenum, dst);
     param->header.samplenum = param->samplenum = dst;
   }
 
