@@ -1,7 +1,5 @@
 /**
  * @file   init_dfa.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 14:20:43 2005
  * 
  * <JA>
  * @brief  DFAʸˡեΥؤɤ߹ߤȥåȥå
@@ -19,13 +17,16 @@
  * find a noise category/word for pause handling.
  * </EN>
  * 
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 14:20:43 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -41,23 +42,25 @@
  * @param dinfo [i/o] a blank DFA data
  * @param filename [in] DFA grammar file name
  */
-void
+boolean
 init_dfa(DFA_INFO *dinfo, char *filename)
 {
   FILE *fd;
   
-  j_printerr("Reading in DFA grammar...");
   if ((fd = fopen_readfile(filename)) == NULL) {
-    j_error("failed to open %s\n",filename);
+    jlog("Error: init_dfa: failed to open %s\n",filename);
+    return FALSE;
   }
   if (!rddfa(fd, dinfo)) {
-    j_error("error in reading %s\n",filename);
+    jlog("Error; init_dfa: error in reading %s\n",filename);
+    return FALSE;
   }
   if (fclose_readfile(fd) == -1) {
-    j_error("close error\n");
+    jlog("Error: init_dfa: failed to close %s\n", filename);
+    return FALSE;
   }
 
-  j_printerr("done\n");
+  return TRUE;
 }
 
 /** 
@@ -67,30 +70,30 @@ init_dfa(DFA_INFO *dinfo, char *filename)
  * @param dinfo [i/o] DFA grammar, category information will be built here.
  * @param winfo [i/o] Word dictionary, word-to-category information will be build here.
  */
-void
+boolean
 make_dfa_voca_ref(DFA_INFO *dinfo, WORD_INFO *winfo)
 {
   WORD_ID i;
-  boolean okflag = TRUE;
+  boolean ok_flag = TRUE;
 
-  j_printerr("Mapping dict item <-> DFA terminal (category)...");
   /* word -> terminal symbol */
   for (i = 0; i < winfo->num; i++) {
     winfo->wton[i] = dfa_symbol_lookup(dinfo, winfo->wname[i]);
     if (winfo->wton[i] == WORD_INVALID) {
       /* error: not found */
-      j_printerr("Error: no such terminal symbol \"%s\" in DFA grammar:\n",
+      jlog("Error: init_dfa: no such terminal symbol \"%s\" in DFA grammar\n",
 	     winfo->wname[i]);
-      put_voca(winfo, i);
-      okflag = FALSE;
+      put_voca(jlog_get_fp(), winfo, i);
+      ok_flag = FALSE;
     }
   }
-  if (!okflag) j_error("Error in dict <-> DFA mapping\n");
 
-  /* terminal symbol -> word */
-  make_terminfo(&(dinfo->term), dinfo, winfo);
-  
-  j_printerr("done\n");
+  if (ok_flag) {
+    /* terminal symbol -> word */
+    make_terminfo(&(dinfo->term), dinfo, winfo);
+  }
+
+  return ok_flag;
 }
 
 /** 
@@ -131,14 +134,15 @@ dfa_find_pause_word(DFA_INFO *dfa, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo)
  * @param src [in] DFA grammar to be appended to @a dst
  * @param coffset appending category point in @a dst
  */
-void
+boolean
 dfa_pause_word_append(DFA_INFO *dst, DFA_INFO *src, int coffset)
 {
   int i;
   /* dst info must be already appended */
   /* [coffset..dst->term_num-1] is the new categories */
   if (dst->term_num - coffset != src->term_num) {
-    j_error("InternalError: appended term num not match!\n");
+    jlog("Error: init_dfa: appended term num not match!\n");
+    return FALSE;
   }
   
   if (dst->is_sp == NULL) {
@@ -154,4 +158,52 @@ dfa_pause_word_append(DFA_INFO *dst, DFA_INFO *src, int coffset)
       dst->sp_id = src->sp_id;
     }
   }
+
+  return TRUE;
+}
+
+/** 
+ * Read grammar (DFA and dictionary) from socket and returns newly allocated
+ * grammars.
+ * 
+ * @param sd [in] socket descpriter
+ * @param ret_dfa [out] read DFA
+ * @param ret_winfo [out] read dictionary
+ * @param hmminfo [in] HMM definition
+ * 
+ * @return TRUE on success, or FALSE on failure.
+ * </EN>
+ */
+boolean
+read_grammar_from_socket(int sd, DFA_INFO **ret_dfa, WORD_INFO **ret_winfo, HTK_HMM_INFO *hmminfo)
+{
+  DFA_INFO *dfa;
+  WORD_INFO *winfo;
+
+  /* load grammar: dfa and dict in turn */
+  dfa = dfa_info_new();
+  if (!
+#ifdef WINSOCK
+      rddfa_sd(sd, dfa)
+#else
+      rddfa_fd(sd, dfa)
+#endif
+      ) {
+    return FALSE;
+  }
+  winfo = word_info_new();
+  /* ignore MONOTREE */
+  if (!
+#ifdef WINSOCK
+      voca_load_htkdict_sd(sd, winfo, hmminfo, FALSE)
+#else
+      voca_load_htkdict_fd(sd, winfo, hmminfo, FALSE)
+#endif
+      ) {
+    dfa_info_free(dfa);
+    return FALSE;
+  }
+  *ret_dfa = dfa;
+  *ret_winfo = winfo;
+  return TRUE;
 }
