@@ -1,7 +1,5 @@
 /**
  * @file   cdset.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 17:58:54 2005
  * 
  * <JA>
  * @brief  Pseudo %HMM åȤκȴ
@@ -59,13 +57,16 @@
  * 
  * </EN>
  *
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 17:58:54 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -113,7 +114,7 @@ cdset_lookup(HTK_HMM_INFO *hmminfo, char *cdstr)
 {
   CD_Set *cd;
   cd = aptree_search_data(cdstr, hmminfo->cdset_info.cdtree);
-  if (strmatch(cdstr, cd->name)) {
+  if (cd != NULL && strmatch(cdstr, cd->name)) {
     return cd;
   } else {
     return NULL;
@@ -175,7 +176,7 @@ put_cdset(void *ptr)
     }
     /*
       for(j=0;j<a->stateset[i].num;j++) {
-        put_htk_state(a->stateset[i].s[j]);
+        put_htk_state(stdout, a->stateset[i].s[j]);
       }
     */
   }
@@ -212,14 +213,16 @@ regist_cdset(APATNODE **root, HTK_HMM_Data *d, char *cdname)
   boolean changed = FALSE;
 
   if (strlen(cdname) >= MAX_HMMNAME_LEN) {
-    j_error("Error: HMM name exceeds limit (%d): %s!\n", MAX_HMMNAME_LEN, cdname);
+    jlog("Error: cdset: HMM name exceeds limit (%d): %s!\n", MAX_HMMNAME_LEN, cdname);
+    jlog("Error: cdset: Please increase the value of MAX_HMMNAME_LEN (current = %d)\n", MAX_HMMNAME_LEN);
+    exit(1);
   }
   
   /* check if the cdset already exist */
   need_new = TRUE;
   if (*root != NULL) {
     lmatch = aptree_search_data(cdname, *root);
-    if (strmatch(lmatch->name, cdname)) {
+    if (lmatch != NULL && strmatch(lmatch->name, cdname)) {
       /* exist, add to it later */
       lset = lmatch;
       need_new = FALSE;
@@ -278,14 +281,13 @@ regist_cdset(APATNODE **root, HTK_HMM_Data *d, char *cdname)
     changed = TRUE;
   }
     
-  /*j_printerr("add to \"%s\"\n", lset->name);*/
   /* register each HMM states to the lcdset */
   for (j=1;j<d->state_num-1;j++) {
     tmp = &(lset->stateset[j]);
     /* check if the state has already registered */
     for(n = 0; n < tmp->num ; n++) {
       if (tmp->s[n] == d->s[j]) { /* compare by pointer */
-	/*j_printerr("\tstate %d has same\n", n);*/
+	/*jlog("\tstate %d has same\n", n);*/
 	break;
       }
     }
@@ -322,8 +324,8 @@ remove_cdset(HTK_HMM_INFO *hmminfo, char *cdname)
   if (hmminfo->cdset_info.cdtree == NULL) return TRUE;
 
   lmatch = aptree_search_data(cdname, hmminfo->cdset_info.cdtree);
-  if (strmatch(lmatch->name, cdname)) {
-    printf("[%s] found\n", lmatch->name);
+  if (lmatch != NULL && strmatch(lmatch->name, cdname)) {
+    jlog("Stat: cdset: [%s] found, removed from cdset\n", lmatch->name);
     /* found */
     /*
     for(j=1;j<lmatch->state_num-1;j++) {
@@ -382,7 +384,7 @@ make_cdset(HTK_HMM_INFO *hmminfo)
 /** 
  * callback for aptree function to free the content of pseudo phone set.
  * 
- * @param d [in] pointer to pseudo phone set in the index tree
+ * @param arg [in] pointer to the pseudo phone set to be free
  */
 static void
 callback_free_lcdset_content(void *arg)
