@@ -1,7 +1,5 @@
 /**
  * @file   chkhmmlist.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 19:17:51 2005
  * 
  * <JA>
  * @brief  HMMList Ƥβǽʥȥ饤ե󤬴ޤޤƤ뤫å
@@ -25,13 +23,16 @@
  * dictionary will be considered.
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 19:17:51 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -58,7 +59,7 @@ make_hmm_basephone_list(HTK_HMM_INFO *hmminfo)
     center_name(lg->name, p);
     if (root != NULL) {
       match = aptree_search_data(p, root);
-      if (strmatch(match->name, p)) continue;
+      if (match != NULL && strmatch(match->name, p)) continue;
     }
     new = (BASEPHONE *)mybmalloc2(sizeof(BASEPHONE), &(hmminfo->mroot));
     new->bgnflag = FALSE;
@@ -82,7 +83,7 @@ static void
 print_callback_detail(void *x)
 {
   BASEPHONE *b = x;
-  j_printf("\"%s\": bgn=%d, end=%d\n", b->name, b->bgnflag, b->endflag);
+  printf("\"%s\": bgn=%d, end=%d\n", b->name, b->bgnflag, b->endflag);
 }
 
 /** 
@@ -94,7 +95,7 @@ static void
 print_callback_name(void *x)
 {
   BASEPHONE *b = x;
-  j_printf("%s, ", b->name);
+  printf("%s, ", b->name);
 }
 /** 
  * Output all basephone informations to stdout.
@@ -115,7 +116,7 @@ void
 print_all_basephone_name(HMM_basephone *base)
 {
   aptree_traverse_and_do(base->root, print_callback_name);
-  j_printf("\n");
+  printf("\n");
 }
 
 static int bncnt;		///< Count of basephone that can appear at the beginning of sentence
@@ -155,24 +156,26 @@ count_all_phone(HMM_basephone *base)
  * @param winfo [in] word dictinary
  * @param base [in] top basephone data holder
  */
-static void
+static boolean
 mark_word_edge(WORD_INFO *winfo, HMM_basephone *base)
 {
   WORD_ID w;
   static char p[MAX_HMMNAME_LEN];
   char *key;
   BASEPHONE *match;
+  boolean ok_p = TRUE;
 
   /* mark what is at beginning of word (can be right context) */
   for(w=0;w<winfo->num;w++) {
     if (w == winfo->head_silwid) continue;
     key = center_name(winfo->wseq[w][0]->name, p);
     match = aptree_search_data(key, base->root);
-    if (strmatch(match->name, key)) {
+    if (match != NULL && strmatch(match->name, key)) {
       match->bgnflag = TRUE;
     } else {
       /* not found!!! */
-      j_error("InternalError: basephone \"%s\" specified in dict, but not found in HMM\n");
+      jlog("Error: chkhmmlist: basephone \"%s\" used in dictionary not exist in HMM definition\n", key);
+      ok_p = FALSE;
     }
   }
   /* mark what is at end of word (can be left context) */
@@ -180,13 +183,16 @@ mark_word_edge(WORD_INFO *winfo, HMM_basephone *base)
     if (w == winfo->tail_silwid) continue;
     key = center_name(winfo->wseq[w][winfo->wlen[w]-1]->name, p);
     match = aptree_search_data(key, base->root);
-    if (strmatch(match->name, key)) {
+    if (match != NULL && strmatch(match->name, key)) {
       match->endflag = TRUE;
     } else {
       /* not found!!! */
-      j_error("InternalError: basephone \"%s\" specified in dict, but not found in HMM\n");
+      jlog("Error: chkhmmlist: basephone \"%s\" used in dictionary not exist in HMM definition\n", key);
+      ok_p = FALSE;
     }
   }
+
+  return ok_p;
 }
 
 
@@ -205,6 +211,7 @@ static int error_num;		///< Number of encountered error phone
  * Add unknown (error) triphone to error list.
  * 
  * @param lostname [in] name of error triphone.
+ * @param hmminfo [i/o] %HMM definition data
  */
 static void
 add_to_error(char *lostname, HTK_HMM_INFO *hmminfo)
@@ -212,7 +219,7 @@ add_to_error(char *lostname, HTK_HMM_INFO *hmminfo)
   char *match = NULL, *new;
   if (error_root != NULL) {
     match = aptree_search_data(lostname, error_root);
-    if (strmatch(match, lostname)) return;
+    if (match != NULL && strmatch(match, lostname)) return;
   }
   new = (char *)mybmalloc2(strlen(lostname)+1, &(hmminfo->mroot));
   strcpy(new, lostname);
@@ -231,7 +238,7 @@ static void
 print_error_callback(void *x)
 {
   char *p = x;
-  j_printf("%s\n", p);
+  printf("%s\n", p);
 }
 
 /** 
@@ -255,7 +262,7 @@ triphone_callback_normal(void *x)
     /* printf("checking \"%s\" - \"%s\"\n", b->name, lg->name); */
     if ((found = htk_hmmdata_lookup_logical(local_hmminfo, gbuf)) == NULL) {
       if (lg->is_pseudo) {
-	j_printerr("Error: \"%s\" not found, fallback to pseudo {%s}\n", gbuf, lg->name);
+	printf("Error: chkhmmlist: \"%s\" not found, fallback to pseudo {%s}\n", gbuf, lg->name);
 	add_to_error(gbuf, local_hmminfo);
       }
     }
@@ -267,7 +274,7 @@ triphone_callback_normal(void *x)
     /* printf("checking \"%s\" - \"%s\"\n", lg->name, b->name); */
     if ((found = htk_hmmdata_lookup_logical(local_hmminfo, gbuf)) == NULL) {
       if (lg->is_pseudo) {
-	j_printerr("Error: \"%s\" not found, fallback to pseudo {%s}\n", gbuf, lg->name);
+	printf("Error: chkhmmlist: \"%s\" not found, fallback to pseudo {%s}\n", gbuf, lg->name);
 	add_to_error(gbuf, local_hmminfo);
       }
     }
@@ -298,7 +305,7 @@ triphone_callback_right(void *x)
     /* printf("	   checking \"%s\" - \"%s\"\n", gbuf, b->name); */
     if ((found = htk_hmmdata_lookup_logical(local_hmminfo, buf)) == NULL) {
       if (lg->is_pseudo) {
-	j_printerr("Error: \"%s\" not found, fallback to pseudo {%s}\n", buf, lg->name);
+	printf("Error: chkhmmlist: \"%s\" not found, fallback to pseudo {%s}\n", buf, lg->name);
 	add_to_error(buf, local_hmminfo);
       }
     }
@@ -323,7 +330,6 @@ triphone_callback_left(void *x)
     lg = local_winfo->wseq[w][0];
     strcpy(gbuf, lg->name);
     add_left_context(gbuf, b->name);
-    /*printf("continue checking \"%s\" - \"%s\"\n", b->name, lg->name);*/
     aptree_traverse_and_do(local_root, triphone_callback_right);
   }
 }
@@ -345,7 +351,7 @@ test_interword_triphone(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
   error_root = NULL;
   error_num = 0;
 
-  j_printf("Inter-word triphone existence test...\n");
+  printf("Inter-word triphone existence test...\n");
   for(w=0;w<winfo->num;w++) {
     current_w = w;
     if (winfo->wlen[w] > 1) {
@@ -357,11 +363,11 @@ test_interword_triphone(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
     }
   }
   if (error_root == NULL) {
-    j_printf("passed\n");
+    printf("passed\n");
   } else {
-    j_printf("following triphones are missing in HMMList:\n");
+    printf("following triphones are missing in HMMList:\n");
     aptree_traverse_and_do(error_root, print_error_callback);
-    j_printf("total %d missing inter-word triphones\n", error_num);
+    printf("total %d missing inter-word triphones\n", error_num);
   }
 }
 
@@ -376,39 +382,42 @@ test_interword_triphone(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
  * @param hmminfo [i/o] %HMM definition information, basephone list will be added.
  * @param winfo [in] word dictionary information
  */
-void
+boolean
 make_base_phone(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
 {
   /* gather base phones and word-{head,tail} phones */
-  j_printf("Exploring HMM database and lexicon tree:\n");
-  mark_word_edge(winfo, &(hmminfo->basephone));
+  jlog("Stat: chkhmmlist: Exploring HMM database and lexicon tree:\n");
+  if (mark_word_edge(winfo, &(hmminfo->basephone)) == FALSE) {
+    return FALSE;
+  }
   count_all_phone(&(hmminfo->basephone));
+  return TRUE;
 }
 
 /** 
  * Output general information concerning phone mapping in %HMM definition.
  * 
+ * @param fp [in] file descriptor
  * @param hmminfo [in] %HMM definition data.
  */
 void
-print_phone_info(HTK_HMM_INFO *hmminfo)
+print_phone_info(FILE *fp, HTK_HMM_INFO *hmminfo)
 {
   /* output information */
-  j_printf("%5d physical HMMs defined in hmmdefs\n", hmminfo->totalhmmnum);
+  fprintf(fp, "%5d physical HMMs defined in hmmdefs\n", hmminfo->totalhmmnum);
   if (hmminfo->totalhmmnum == hmminfo->totallogicalnum - hmminfo->totalpseudonum) {
-    j_printf("   no HMMList, physical HMM names are redirected to logicalHMM\n");
+    fprintf(fp, "   no HMMList, physical HMM names are redirected to logicalHMM\n");
   } else {
     if (hmminfo->is_triphone) {
-      j_printf("%5d triphones listed in hmmlist\n", hmminfo->totallogicalnum - hmminfo->totalpseudonum);
+      fprintf(fp, "%5d triphones listed in hmmlist\n", hmminfo->totallogicalnum - hmminfo->totalpseudonum);
     } else {
-      j_printf("%5d phones in hmmlist\n", hmminfo->totallogicalnum - hmminfo->totalpseudonum);
+      fprintf(fp, "%5d phones in hmmlist\n", hmminfo->totallogicalnum - hmminfo->totalpseudonum);
     }
   }
   if (hmminfo->totalpseudonum != 0) {
-    j_printf("%5d pseudo HMM generated for missing mono/bi-phones\n",hmminfo->totalpseudonum);
+    fprintf(fp, "%5d pseudo HMM generated for missing mono/bi-phones\n",hmminfo->totalpseudonum);
   }
-  j_printf("%5d TOTAL logical HMMs\n", hmminfo->totallogicalnum);
-  j_printf("%5d base phones in logical HMM\n", hmminfo->basephone.num);
-  j_printf("%5d phones appear on word head, %d phones on word tail\n", hmminfo->basephone.bgnnum, hmminfo->basephone.endnum);
-
+  fprintf(fp, "%5d TOTAL logical HMMs\n", hmminfo->totallogicalnum);
+  fprintf(fp, "%5d base phones in logical HMM\n", hmminfo->basephone.num);
+  fprintf(fp, "%5d phones appear on word head, %d phones on word tail\n", hmminfo->basephone.bgnnum, hmminfo->basephone.endnum);
 }
