@@ -1,7 +1,5 @@
 /**
  * @file   hmm_lookup.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 22:34:30 2005
  * 
  * <JA>
  * @brief  %HMM ̾Τ򸡺
@@ -33,13 +31,16 @@
  * to the logical %HMM mapping function is also done here.
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 22:34:30 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -63,7 +64,7 @@ htk_hmmdata_lookup_physical(HTK_HMM_INFO *hmminfo, char *keyname)
 {
   HTK_HMM_Data *tmp;
   tmp = aptree_search_data(keyname, hmminfo->physical_root);
-  if (strmatch(tmp->name, keyname)) {
+  if (tmp != NULL && strmatch(tmp->name, keyname)) {
     return tmp;
   } else {
     return NULL;
@@ -83,7 +84,7 @@ htk_hmmdata_lookup_logical(HTK_HMM_INFO *hmminfo, char *keyname)
 {
   HMM_Logical *tmp;
   tmp = aptree_search_data(keyname, hmminfo->logical_root);
-  if (strmatch(tmp->name, keyname)) {
+  if (tmp != NULL && strmatch(tmp->name, keyname)) {
     return tmp;
   } else {
     return NULL;
@@ -126,9 +127,9 @@ hmm_add_physical_to_logical(HTK_HMM_INFO *hmminfo)
     /* check if same name already exist */
     if (hmminfo->logical_root != NULL) {
       match = aptree_search_data(ph->name, hmminfo->logical_root);
-      if (strmatch(match->name, ph->name)) {
+      if (match != NULL && strmatch(match->name, ph->name)) {
 	/* the physcal name was already mapped to other HMMs in HMMList */
-	j_printerr("Warning: \"%s\" is defined in hmmdefs, but \"%s\" will be used instead\n", ph->name, (match->body.defined)->name);
+	jlog("Warning: hmm_lookup: \"%s\" is defined in hmmdefs, but \"%s\" will be used instead\n", ph->name, (match->body.defined)->name);
 	continue;
       }
     }
@@ -165,14 +166,14 @@ static int add_count;		///< Number of pseudo phones added to logical %HMM list.
  * @param hmminfo [in] %HMM definition data.
  * @param name [in] name of the pseudo phone to add.
  */
-static void
+static boolean
 hmm_add_pseudo_phones_sub(HTK_HMM_INFO *hmminfo, char *name)
 {
   HMM_Logical *new, *match;
 
   /* check if already exist */
   match = aptree_search_data(name, hmminfo->logical_root);
-  if (strmatch(match->name, name)) {
+  if (match != NULL && strmatch(match->name, name)) {
     /* already exist in list */
     /*    if (! match->is_pseudo) {*/
       /* this pseudo-HMM is already defined as real HMM in hmmdefs or in HMMList */
@@ -186,7 +187,8 @@ hmm_add_pseudo_phones_sub(HTK_HMM_INFO *hmminfo, char *name)
     new->is_pseudo = TRUE;
     new->body.pseudo = cdset_lookup(hmminfo, name);
     if (new->body.pseudo == NULL) {	/* should never happen */
-      j_error("InternalError: tried to add pseudo phone \"%s\" to logical HMM, but no corresponding CD_Set found.  Why??\n");
+      jlog("Error: hmm_lookup: tried to add pseudo phone \"%s\" to logical HMM, but no corresponding CD_Set found.  Why??\n");
+      return FALSE;
     }
     new->next = hmminfo->lgstart;
     hmminfo->lgstart = new;
@@ -197,6 +199,7 @@ hmm_add_pseudo_phones_sub(HTK_HMM_INFO *hmminfo, char *name)
     }
     add_count++;
   }
+  return TRUE;
 }
     
 /** 
@@ -210,25 +213,34 @@ hmm_add_pseudo_phones(HTK_HMM_INFO *hmminfo)
 {
   HMM_Logical *lg;
   static char buf[MAX_HMMNAME_LEN];
+  boolean ok_p = TRUE;
 
   add_count = 0;
   /* add pseudo monophone */
   for (lg = hmminfo->lgstart; lg; lg = lg->next) {
     if (lg->is_pseudo) continue;
-    hmm_add_pseudo_phones_sub(hmminfo, center_name(lg->name, buf));
+    if (hmm_add_pseudo_phones_sub(hmminfo, center_name(lg->name, buf)) == FALSE) {
+      jlog("Error: hmm_lookup: failed to add \"%s\" as logical\n", center_name(lg->name, buf));
+      ok_p = FALSE;
+    }
   }
   /* add pseudo biphone, i.e. "a-k" etc. */
   for (lg = hmminfo->lgstart; lg; lg = lg->next) {
     if (lg->is_pseudo) continue;
-    hmm_add_pseudo_phones_sub(hmminfo, leftcenter_name(lg->name, buf));
+    if (hmm_add_pseudo_phones_sub(hmminfo, leftcenter_name(lg->name, buf)) == FALSE) {
+      jlog("Error: hmm_lookup: failed to add \"%s\" as logical\n", leftcenter_name(lg->name, buf));
+      ok_p = FALSE;
+    }
   }
   /* add pseudo biphone, i.e. "k+e" etc. */
   for (lg = hmminfo->lgstart; lg; lg = lg->next) {
     if (lg->is_pseudo) continue;
-    hmm_add_pseudo_phones_sub(hmminfo, rightcenter_name(lg->name, buf));
+    if (hmm_add_pseudo_phones_sub(hmminfo, rightcenter_name(lg->name, buf)) == FALSE) {
+      jlog("Error: hmm_lookup: failed to add \"%s\" as logical\n", rightcenter_name(lg->name, buf));
+      ok_p = FALSE;
+    }
   }
-  j_printerr("%d added as logical...", add_count);
-
+  jlog("Stat: hmm_lookup: %d pseudo phones are added to logical HMM list\n", add_count);
   hmminfo->totalpseudonum = add_count;
   /* re-count total number */
   hmm_count_logical_num(hmminfo);
