@@ -1,7 +1,5 @@
 /**
  * @file   rdhmmdef.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 00:17:18 2005
  * 
  * <JA>
  * @brief  HTK %HMM եɤ߹ߡᥤ
@@ -29,13 +27,16 @@
  * variable to read the current token.
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 00:17:18 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -52,7 +53,7 @@ static int line;		///< Input Line count
 /* global functions for rdhmmdef_*.c */
 
 /** 
- * Output error message, with current reading status.
+ * Output error message, with current reading status, and terminate
  * 
  * @param str [in] error string
  */
@@ -60,10 +61,12 @@ void
 rderr(char *str)
 {
   if (rdhmmdef_token == NULL) {	/* end of file */
-    j_error("\nError: %s on end of file\n", str);
+    jlog("Error: rdhmmdef: %s on end of file\n", str);
   } else {
-    j_error("\nError at line %d: %s\n", line, (str) ? str : "parse error");
+    jlog("Error: rdhmmdef: read error at line %d: %s\n", line, (str) ? str : "parse error");
   }
+  jlog_flush();
+  exit(1);
 }
 
 /** 
@@ -118,6 +121,24 @@ conv_log_arc(HTK_HMM_INFO *hmm)
     }
   }
 }
+/** 
+ * Invert all the variance values.
+ * 
+ * @param hmm [i/o] %HMM definition data to modify.
+ */
+void
+htk_hmm_inverse_variances(HTK_HMM_INFO *hmm)
+{
+  HTK_HMM_Var *v;
+  int i;
+
+  for (v = hmm->vrstart; v; v = v->next) {
+    for(i=0;i<v->len;i++) {
+      v->vec[i] = 1 / v->vec[i];
+    }
+  }
+}
+
 
 /** 
  * @brief  Main top routine to read in HTK %HMM definition file.
@@ -136,6 +157,9 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
   char macrosw;
   char *name;
 
+  /* variances in htkdefs are not inversed yet */
+  hmm->variance_inversed = FALSE;
+
   /* read the first token */
   read_token(fp);
   
@@ -189,20 +213,37 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
     }
   }
 
-  j_printerr("(ascii)...");
+  /* convert transition prob to log scale */
+  conv_log_arc(hmm);
+
+  jlog("Stat: rdhmmdef: ascii format HMM definition\n");
   
   /* check limitation */
   if (check_all_hmm_limit(hmm)) {
-    j_printerr("limit check passed\n");
+    jlog("Stat: rdhmmdef: limit check passed\n");
   } else {
-    j_error("Error: cannot use this HMM for system limitation.\n");
+    jlog("Error: rdhmmdef: cannot handle this HMM due to system limitation\n");
+    return FALSE;
+  }
+
+  /* determine whether this model needs multi-path handling */
+  hmm->need_multipath = htk_hmm_has_several_arc_on_edge(hmm);
+  if (hmm->need_multipath) {
+    jlog("Stat: rdhmmdef: this HMM requires multipath handling at decoding\n");
+  } else {
+    jlog("Stat: rdhmmdef: this HMM does not need multipath handling\n");
   }
-  /* convert transition prob to log scale */
-  conv_log_arc(hmm);
   
+  /* inverse all variance values for faster computation */
+  if (! hmm->variance_inversed) {
+    htk_hmm_inverse_variances(hmm);
+    hmm->variance_inversed = TRUE;
+  }
+
   /* check HMM parameter option type */
   if (!check_hmm_options(hmm)) {
-    j_error("hmm options check failed\n");
+    jlog("Error: rdhmmdef: hmm options check failed\n");
+    return FALSE;
   }
 
   /* add ID number for all HTK_HMM_State */
@@ -216,7 +257,9 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
       if (max < stmp->mix_num) max = stmp->mix_num;
       stmp->id = n++;
       if (n >= MAX_STATE_NUM) {
-	j_error("Error: too much states > %d\n", MAX_STATE_NUM);
+	jlog("Error: rdhmmdef: too much states in a model > %d\n", MAX_STATE_NUM);
+	jlog("Error: you can try changing value of MAX_STATE_NUM\n");
+	return FALSE;
       }
     }
     hmm->totalstatenum = n;
