@@ -1,7 +1,5 @@
 /**
  * @file   rdhmmdef_trans.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 03:50:55 2005
  * 
  * <JA>
  * @brief  HTK %HMM եɤ߹ߡܹ
@@ -16,13 +14,16 @@
  * after finished reading the while %HMM definition file.
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 03:50:55 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -71,8 +72,8 @@ trans_add(HTK_HMM_INFO *hmm, HTK_HMM_Trans *new)
       hmm->tr_root = aptree_make_root_node(new);
     } else {
       match = aptree_search_data(new->name, hmm->tr_root);
-      if (strmatch(match->name,new->name)) {
-	j_printerr("Error: ~t \"%s\" is already defined\n", new->name);
+      if (match != NULL && strmatch(match->name,new->name)) {
+	jlog("Error: rdhmmdef_trans: ~t \"%s\" is already defined\n", new->name);
 	rderr(NULL);
       } else {
 	aptree_add_entry(new->name, new, match->name, &(hmm->tr_root));
@@ -95,7 +96,7 @@ trans_lookup(HTK_HMM_INFO *hmm, char *keyname)
   HTK_HMM_Trans *t;
 
   t = aptree_search_data(keyname, hmm->tr_root);
-  if (strmatch(t->name, keyname)) {
+  if (t != NULL && strmatch(t->name, keyname)) {
     return t;
   } else {
     return NULL;
@@ -112,6 +113,7 @@ trans_lookup(HTK_HMM_INFO *hmm, char *keyname)
  * the pointer to the found component will be assigned to this model.
  * 
  * @param fp [in] file pointer
+ * @param hmm [i/o] %HMM definition data
  * 
  * @return pointer to the newly read data.
  */
@@ -184,7 +186,7 @@ get_trans_data(FILE *fp, HTK_HMM_INFO *hmm)
     NoTokErr("missing TRANSP macro name");
     tmp = trans_lookup(hmm, rdhmmdef_token);
     if (tmp == NULL) {
-      j_printerr("~t \"%s\" not defined\n", rdhmmdef_token);
+      jlog("Error: rdhmmdef_trans: ~t \"%s\" not defined\n", rdhmmdef_token);
       rderr(NULL);
     }
     read_token(fp);
