@@ -1,7 +1,5 @@
 /**
  * @file   write_binhmm.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 06:03:36 2005
  * 
  * <JA>
  * @brief  %HMM ХʥΥեؽ񤭽Ф
@@ -21,20 +19,27 @@
  * not compatible with the HTK binary format.
  * </EN>
  * 
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 06:03:36 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
  * Copyright (c) 2003-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* $Id: write_binhmm.c,v 1.2 2006/03/28 07:29:01 sumomo Exp $ */
+/* $Id: write_binhmm.c,v 1.2 2007/12/18 08:45:51 sumomo Exp $ */
 
 #include <sent/stddefs.h>
 #include <sent/htk_param.h>
 #include <sent/htk_hmm.h>
+#include <sent/mfcc.h>
+
+#define wrt(A,B,C,D) if (wrtfunc(A,B,C,D) == FALSE) return FALSE
+#define wrt_str(A,B) if (wrt_strfunc(A,B) == FALSE) return FALSE
 
 
 /** 
@@ -45,8 +50,8 @@
  * @param unitbyte [in] size of a unit in bytes
  * @param unitnum [in] number of unit to write
  */
-static void
-wrt(FILE *fp, void *buf, size_t unitbyte, int unitnum)
+static boolean
+wrtfunc(FILE *fp, void *buf, size_t unitbyte, size_t unitnum)
 {
 #ifndef WORDS_BIGENDIAN
   if (unitbyte != 1) {
@@ -54,14 +59,15 @@ wrt(FILE *fp, void *buf, size_t unitbyte, int unitnum)
   }
 #endif
   if (myfwrite(buf, unitbyte, unitnum, fp) < unitnum) {
-    perror("write_binhmm: wrt");
-    j_error("write failed\n");
+    jlog("Error: write_binhmm: failed to write %d bytes", unitbyte * unitnum);
+    return FALSE;
   }
 #ifndef WORDS_BIGENDIAN
   if (unitbyte != 1) {
     swap_bytes((char *)buf, unitbyte, unitnum);
   }
 #endif
+  return TRUE;
 }
 
 /** 
@@ -70,29 +76,83 @@ wrt(FILE *fp, void *buf, size_t unitbyte, int unitnum)
  * @param fp [in] file pointer
  * @param str [in] string to write
  */
-static void
-wrt_str(FILE *fp, char *str)
+static boolean
+wrt_strfunc(FILE *fp, char *str)
 {
   static char noname = '\0';
+  boolean ret;
   
   if (str) {
-    wrt(fp, str, sizeof(char), strlen(str)+1);
+    ret = wrtfunc(fp, str, sizeof(char), strlen(str)+1);
   } else {
-    wrt(fp, &noname, sizeof(char), 1);
+    ret = wrtfunc(fp, &noname, sizeof(char), 1);
   }
+  return ret;
 }
 
 
-static char *binhmm_header = BINHMM_HEADER; ///< Header string
+static char *binhmm_header_v2 = BINHMM_HEADER_V2; ///< Header string for V2
+
 /** 
- * Write header string as binary HMM file.
+ * Write header string as binary HMM file (ver. 2)
  * 
  * @param fp [in] file pointer
  */
-static void
-wt_header(FILE *fp)
+static boolean
+wt_header(FILE *fp, boolean emp, boolean inv)
 {
-  wrt_str(fp, binhmm_header);
+  char buf[50];
+  char *p;
+
+  wrt_str(fp, binhmm_header_v2);
+  p = &(buf[0]);
+  if (emp) {
+    *p++ = '_';
+    *p++ = BINHMM_HEADER_V2_EMBEDPARA;
+  }
+  if (inv) {
+    *p++ = '_';
+    *p++ = BINHMM_HEADER_V2_VARINV;
+  }
+  *p = '\0';
+  wrt_str(fp, buf);
+
+  return TRUE;
+}
+
+
+/** 
+ * Write acoustic analysis configration parameters into header of binary HMM.
+ * 
+ * @param fp [in] file pointer
+ * @param para [in] acoustic analysis configration parameters
+ */
+static boolean
+wt_para(FILE *fp, Value *para)
+{
+  short version;
+
+  version = VALUE_VERSION;
+  wrt(fp, &version, sizeof(short), 1);
+
+  wrt(fp, &(para->smp_period), sizeof(long), 1);      
+  wrt(fp, &(para->smp_freq), sizeof(long), 1);	
+  wrt(fp, &(para->framesize), sizeof(int), 1);        
+  wrt(fp, &(para->frameshift), sizeof(int), 1);       
+  wrt(fp, &(para->preEmph), sizeof(float), 1);        
+  wrt(fp, &(para->lifter), sizeof(int), 1);           
+  wrt(fp, &(para->fbank_num), sizeof(int), 1);        
+  wrt(fp, &(para->delWin), sizeof(int), 1);           
+  wrt(fp, &(para->accWin), sizeof(int), 1);           
+  wrt(fp, &(para->silFloor), sizeof(float), 1);       
+  wrt(fp, &(para->escale), sizeof(float), 1);         
+  wrt(fp, &(para->hipass), sizeof(int), 1);		
+  wrt(fp, &(para->lopass), sizeof(int), 1);		
+  wrt(fp, &(para->enormal), sizeof(int), 1);          
+  wrt(fp, &(para->raw_e), sizeof(int), 1);            
+  wrt(fp, &(para->zmeanframe), sizeof(int), 1);	
+
+  return TRUE;
 }
 
 
@@ -102,7 +162,7 @@ wt_header(FILE *fp)
  * @param fp [in] file pointer
  * @param opt [out] pointer to the %HMM option structure that holds the values.
  */
-static void
+static boolean
 wt_opt(FILE *fp, HTK_HMM_Options *opt)
 {
   wrt(fp, &(opt->stream_info.num), sizeof(short), 1);
@@ -111,6 +171,7 @@ wt_opt(FILE *fp, HTK_HMM_Options *opt)
   wrt(fp, &(opt->cov_type), sizeof(short), 1);
   wrt(fp, &(opt->dur_type), sizeof(short), 1);
   wrt(fp, &(opt->param_type), sizeof(short), 1);
+  return TRUE;
 }
 
 /** 
@@ -119,11 +180,12 @@ wt_opt(FILE *fp, HTK_HMM_Options *opt)
  * @param fp [in] file pointer
  * @param hmm [out] pointer to the writing %HMM definition data
  */
-static void
+static boolean
 wt_type(FILE *fp, HTK_HMM_INFO *hmm)
 {
   wrt(fp, &(hmm->is_tied_mixture), sizeof(boolean), 1);
   wrt(fp, &(hmm->maxmixturenum), sizeof(int), 1);
+  return TRUE;
 }
 
 
@@ -159,7 +221,7 @@ qsort_tr_index(HTK_HMM_Trans **t1, HTK_HMM_Trans **t2)
  * @param fp [in] file pointer
  * @param hmm [in] writing %HMM definition data 
  */
-static void
+static boolean
 wt_trans(FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Trans *t;
@@ -183,7 +245,9 @@ wt_trans(FILE *fp, HTK_HMM_INFO *hmm)
     }
   }
 
-  j_printf("%d transition maxtix written\n", tr_num);
+  jlog("Stat: write_binhmm: %d transition maxtix written\n", tr_num);
+
+  return TRUE;
 }
 
 /** 
@@ -244,7 +308,7 @@ qsort_vr_index(HTK_HMM_Var **v1, HTK_HMM_Var **v2)
  * @param fp [in] file pointer
  * @param hmm [in] writing %HMM definition data 
  */
-static void
+static boolean
 wt_var(FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Var *v;
@@ -264,7 +328,9 @@ wt_var(FILE *fp, HTK_HMM_INFO *hmm)
     wrt(fp, &(v->len), sizeof(short), 1);
     wrt(fp, v->vec, sizeof(VECT), v->len);
   }
-  j_printf("%d variance written\n", vr_num);
+  jlog("Stat: write_binhmm: %d variance written\n", vr_num);
+
+  return TRUE;
 }
 
 /** 
@@ -327,7 +393,7 @@ qsort_dens_index(HTK_HMM_Dens **d1, HTK_HMM_Dens **d2)
  * @param fp [in] file pointer
  * @param hmm [in] writing %HMM definition data 
  */
-static void
+static boolean
 wt_dens(FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Dens *d;
@@ -348,11 +414,16 @@ wt_dens(FILE *fp, HTK_HMM_INFO *hmm)
     wrt(fp, d->mean, sizeof(VECT), d->meanlen);
     vid = search_vid(d->var);
     /* for debug */
-    if (d->var != vr_index[vid]) j_error("index not match!!! dens\n");
+    if (d->var != vr_index[vid]) {
+      jlog("Error: write_binhmm: index not match!!!\n");
+      return FALSE;
+    }
     wrt(fp, &vid, sizeof(unsigned int), 1);
     wrt(fp, &(d->gconst), sizeof(LOGPROB), 1);
   }
-  j_printf("%d gaussian densities written\n", dens_num);
+  jlog("Stat: write_binhmm: %d gaussian densities written\n", dens_num);
+
+  return TRUE;
 }
 
 /** 
@@ -429,7 +500,7 @@ qsort_tm_index(GCODEBOOK **tm1, GCODEBOOK **tm2)
  * @param fp [in] file pointer
  * @param hmm [in] writing %HMM definition data 
  */
-static void
+static boolean
 wt_tmix(FILE *fp, HTK_HMM_INFO *hmm)
 {
   GCODEBOOK *tm;
@@ -454,12 +525,17 @@ wt_tmix(FILE *fp, HTK_HMM_INFO *hmm)
       } else {
 	did = search_did(tm->d[i]);
 	/* for debug */
-	if (tm->d[i] != dens_index[did]) j_error("index not match!!! dens\n");
+	if (tm->d[i] != dens_index[did]) {
+	  jlog("Error: write_binhmm: index not match!!!\n");
+	  return FALSE;
+	}
       }
       wrt(fp, &did, sizeof(unsigned int), 1);
     }
   }
-  j_printf("%d tied-mixture codebooks written\n", tm_num);
+  jlog("Stat: write_binhmm: %d tied-mixture codebooks written\n", tm_num);
+
+  return TRUE;
 }
 
 /** 
@@ -522,7 +598,7 @@ qsort_st_index(HTK_HMM_State **s1, HTK_HMM_State **s2)
  * @param fp [in] file pointer
  * @param hmm [in] writing %HMM definition data 
  */
-static void
+static boolean
 wt_state(FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_State *s;
@@ -558,7 +634,8 @@ wt_state(FILE *fp, HTK_HMM_INFO *hmm)
 	  } else {
 	    did = search_did(s->b[i]);
 	    if (s->b[i] != dens_index[did]) {
-	      j_error("index not match!!!");
+	      jlog("Error: write_binhmm: index not match!!!\n");
+	      return FALSE;
 	    }
 	  }
 	  wrt(fp, &did, sizeof(unsigned int), 1);
@@ -572,7 +649,8 @@ wt_state(FILE *fp, HTK_HMM_INFO *hmm)
 	} else {
 	  did = search_did(s->b[i]);
 	  if (s->b[i] != dens_index[did]) {
-	    j_error("index not match!!!");
+	    jlog("Error: write_binhmm: index not match!!!\n");
+	    return FALSE;
 	  }
 	}
 	wrt(fp, &did, sizeof(unsigned int), 1);
@@ -580,7 +658,9 @@ wt_state(FILE *fp, HTK_HMM_INFO *hmm)
     }
     wrt(fp, s->bweight, sizeof(PROB), s->mix_num);
   }
-  j_printf("%d states written\n", st_num);
+  jlog("Stat: write_binhmm: %d states written\n", st_num);
+
+  return TRUE;
 }
 
 /** 
@@ -620,7 +700,7 @@ search_stid(HTK_HMM_State *s)
  * @param fp [in] file pointer
  * @param hmm [in] writing %HMM definition data 
  */
-static void
+static boolean
 wt_data(FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Data *d;
@@ -638,7 +718,10 @@ wt_data(FILE *fp, HTK_HMM_INFO *hmm)
       if (d->s[i] != NULL) {
 	sid = search_stid(d->s[i]);
 	/* for debug */
-	if (d->s[i] != st_index[sid]) j_error("index not match!!! data state\n");
+	if (d->s[i] != st_index[sid]) {
+	  jlog("Error: write_binhmm: index not match!!!\n");
+	  return FALSE;
+	}
       } else {
 	sid = hmm->totalstatenum + 1; /* error value */
       }
@@ -646,10 +729,14 @@ wt_data(FILE *fp, HTK_HMM_INFO *hmm)
     }
     tid = search_trid(d->tr);
     /* for debug */
-    if (d->tr != tr_index[tid]) j_error("index not match!!! data trans\n");
+    if (d->tr != tr_index[tid]) {
+      jlog("Error: write_binhmm: index not match!!!\n");
+      return FALSE;
+    }
     wrt(fp, &tid, sizeof(unsigned int), 1);
   }
-  j_printf("%d HMM model definition written\n", md_num);
+  jlog("Stat: write_binhmm: %d HMM model definition written\n", md_num);
+  return TRUE;
 }
 
 
@@ -658,40 +745,77 @@ wt_data(FILE *fp, HTK_HMM_INFO *hmm)
  * 
  * @param fp [in] file pointer
  * @param hmm [in] %HMM definition structure to be written
+ * @param para [in] acoustic analysis parameter, or NULL if not available
  * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-write_binhmm(FILE *fp, HTK_HMM_INFO *hmm)
+write_binhmm(FILE *fp, HTK_HMM_INFO *hmm, Value *para)
 {
+
   /* write header */
-  wt_header(fp);
+  if (wt_header(fp, (para ? TRUE : FALSE), hmm->variance_inversed) == FALSE) {
+    jlog("Error: write_binhmm: failed to write header\n");
+    return FALSE;
+  }
+
+  if (para) {
+    /* write acoustic analysis parameter info */
+    if (wt_para(fp, para) == FALSE) {
+      jlog("Error: write_binhmm: failed to write acoustic analysis parameters\n");
+      return FALSE;
+    }
+  }
   
   /* write option data */
-  wt_opt(fp, &(hmm->opt));
+  if (wt_opt(fp, &(hmm->opt)) == FALSE) {
+    jlog("Error: write_binhmm: failed to write option data\n");
+    return FALSE;
+  }
 
   /* write type data */
-  wt_type(fp, hmm);
+  if (wt_type(fp, hmm) == FALSE) {
+    jlog("Error: write_binhmm: failed to write HMM type data\n");
+    return FALSE;
+  }
 
   /* write transition data */
-  wt_trans(fp, hmm);
+  if (wt_trans(fp, hmm) == FALSE) {
+    jlog("Error: write_binhmm: failed to write HMM transition data\n");
+    return FALSE;
+  }
 
   /* write variance data */
-  wt_var(fp, hmm);
+  if (wt_var(fp, hmm) == FALSE) {
+    jlog("Error: write_binhmm: failed to write HMM variance data\n");
+    return FALSE;
+  }
 
   /* write density data */
-  wt_dens(fp, hmm);
+  if (wt_dens(fp, hmm) == FALSE) {
+    jlog("Error: write_binhmm: failed to write density data\n");
+    return FALSE;
+  }
 
   /* write tmix data */
   if (hmm->is_tied_mixture) {
-    wt_tmix(fp, hmm);
+    if (wt_tmix(fp, hmm) == FALSE) {
+      jlog("Error: write_binhmm: failed to write tied-mixture codebook data\n");
+      return FALSE;
+    }
   }
 
   /* write state data */
-  wt_state(fp, hmm);
+  if (wt_state(fp, hmm) == FALSE) {
+    jlog("Error: write_binhmm: failed to write HMM state data\n");
+    return FALSE;
+  }
 
   /* write model data */
-  wt_data(fp, hmm);
+  if (wt_data(fp, hmm) == FALSE) {
+    jlog("Error: write_binhmm: failed to write HMM data\n");
+    return FALSE;
+  }
 
   /* free pointer->index work area */
   free(tr_index);
