@@ -1,7 +1,5 @@
 /**
  * @file   ss.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 17:19:54 2005
  * 
  * <JA>
  * @brief  ڥȥ븺
@@ -18,13 +16,16 @@
  * of audio input, and file I/O for that.
  * </EN>
  * 
- * $Revision: 1.4 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 17:19:54 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -69,28 +70,28 @@ new_SS_load_from_file(char *filename, int *slen)
   float *sbuf;
 
   /* open file */
-  j_printerr("Reading Noise Spectrum for SS...");
+  jlog("Stat: ss: reading Noise Spectrum for SS\n");
   if ((fp = fopen_readfile(filename)) == NULL) {
-    j_printerr("Error: SS_load_from_file: cannot open \"%s\"\n", filename);
+    jlog("Error: ss: failed to open \"%s\"\n", filename);
     return(NULL);
   }
   /* read length */
   if (myread(&num, sizeof(int), 1, fp) == FALSE) {
-    j_printerr("Error: SS_load_from_file: cannot read \"%s\"\n", filename);
+    jlog("Error: ss: failed to read \"%s\"\n", filename);
     return(NULL);
   }
   /* allocate */
   sbuf = (float *)mymalloc(sizeof(float) * num);
   /* read data */
   if (myread(sbuf, sizeof(float), num, fp) == FALSE) {
-    j_printerr("Error: SS_load_from_file: cannot read \"%s\"\n", filename);
+    jlog("Error: ss: failed to read \"%s\"\n", filename);
     return(NULL);
   }
   /* close file */
   fclose_readfile(fp);
 
   *slen = num;
-  j_printerr("done\n");
+  jlog("Stat: ss: done\n");
   return(sbuf);
 }
 
@@ -100,83 +101,65 @@ new_SS_load_from_file(char *filename, int *slen)
  * 
  * @param wave [in] input audio data sequence
  * @param wavelen [in] length of above
- * @param para [in] parameter
  * @param slen [out] length of returned buffer
+ * @param w [i/o] MFCC calculation work area
+ * @param para [in] parameter
  * 
  * @return a newly allocated buffer that contains the calculated spectrum.
  */
 float *
-new_SS_calculate(SP16 *wave, int wavelen, Value para, int *slen)
+new_SS_calculate(SP16 *wave, int wavelen, int *slen, MFCCWork *w, Value *para)
 {
-  int fftN, n;
-  float *bf, *Re, *Im, *spec;
+  float *spec;
   int t, framenum, start, end, k, i;
   double x, y;
   
-  /* Calculate FFT size */
-  fftN = 2;  n = 1;
-  while(para.framesize > fftN){
-    fftN *= 2; n++;
-  }
-
-#ifdef MFCC_SINCOS_TABLE
-  /* generate tables */
-  make_costbl_hamming(para.framesize);
-  make_fft_table(n);
-#endif
-  
   /* allocate work area */
-  spec = (float *)mymalloc((fftN + 1) * sizeof(float));
-  bf = (float *)mymalloc((fftN + 1) * sizeof(float));
-  Re = (float *)mymalloc((fftN + 1) * sizeof(float));
-  Im = (float *)mymalloc((fftN + 1) * sizeof(float));
-  for(i=0;i<fftN;i++) spec[i] = 0.0;
+  spec = (float *)mymalloc((w->fb.fftN + 1) * sizeof(float));
+  for(i=0;i<w->fb.fftN;i++) spec[i] = 0.0;
   
-  /* Caluculate Sum of Noise Power Spectral */
-  framenum = (int)((wavelen - para.framesize) / para.frameshift) + 1;
+  /* Caluculate sum of noise power spectrum */
+  framenum = (int)((wavelen - para->framesize) / para->frameshift) + 1;
   start = 1;
   end = 0;
   for (t = 0; t < framenum; t++) {
-    if (end != 0) start = end - (para.framesize - para.frameshift) - 1;
+    if (end != 0) start = end - (para->framesize - para->frameshift) - 1;
     k = 1;
-    for (i = start; i <= start + para.framesize; i++) {
-      bf[k] = (float)wave[i-1];
+    for (i = start; i <= start + para->framesize; i++) {
+      w->bf[k] = (float)wave[i-1];
       k++;
     }
     end = i;
 
-    if (para.zmeanframe) {
-      ZMeanFrame(bf, para.framesize);
+    if (para->zmeanframe) {
+      ZMeanFrame(w->bf, para->framesize);
     }
 
     /* Pre-emphasis */
-    PreEmphasise(bf, para);
+    PreEmphasise(w->bf, para->framesize, para->preEmph);
     /* Hamming Window */
-    Hamming(bf, para.framesize);
+    Hamming(w->bf, para->framesize, w);
     /* FFT Spectrum */
-    for (i = 1; i <= para.framesize; i++) {
-      Re[i-1] = bf[i]; Im[i-1] = 0.0;
+    for (i = 1; i <= para->framesize; i++) {
+      w->fb.Re[i-1] = w->bf[i]; w->fb.Im[i-1] = 0.0;
     }
-    for (i = para.framesize + 1; i <= fftN; i++) {
-      Re[i-1] = 0.0;   Im[i-1] = 0.0;
+    for (i = para->framesize + 1; i <= w->fb.fftN; i++) {
+      w->fb.Re[i-1] = 0.0;   w->fb.Im[i-1] = 0.0;
     }
-    FFT(Re, Im, n);
+    FFT(w->fb.Re, w->fb.Im, w->fb.n, w);
     /* Sum noise spectrum */
-    for(i = 1; i <= fftN; i++){
-      x = Re[i - 1];  y = Im[i - 1];
+    for(i = 1; i <= w->fb.fftN; i++){
+      x = w->fb.Re[i - 1];  y = w->fb.Im[i - 1];
       spec[i - 1] += sqrt(x * x + y * y);
     }
   }
 
   /* Calculate average noise spectrum */
-  for(t=0;t<fftN;t++) {
+  for(t=0;t<w->fb.fftN;t++) {
     spec[t] /= (float)framenum;
   }
 
   /* return the new spec[] */
-  free(Im);
-  free(Re);
-  free(bf);
-  *slen = fftN;
+  *slen = w->fb.fftN;
   return(spec);
 }
